"use strict";
angular.module ('glDatePickerFactory', [])
  .factory ('glDatePickerFactory', ['$rootScope', '$filter',
  function ($rootScope, $filter) {
    return {
      /**
       * @description checks if selected start and end date correspond to the default values and set the scope.defaultDates
       * accordingly
       *
       * @param {Object} scope - the controller scope
       */
      checkDefaultDates: function (scope) {
        if (scope.settings && scope.settings.defaults) {
          var defaultStartDate = this.getDateFromRelative(scope.settings.defaults[0]);
          var defaultEndDate = this.getDateFromRelative(scope.settings.defaults[1]);
          if (!defaultStartDate || !defaultEndDate) {
            scope.defaultDates = true;
          } else {
            if (this.compareDates(scope.startDate, defaultStartDate) === 0 &&
              this.compareDates(scope.endDate, defaultEndDate) === 0) {
              scope.defaultDates = true;
            } else {
              scope.defaultDates = false;
            }
          }
        } else {
          scope.defaultDates = true;
        }
      },

      /**
       * @description it creates the object representing the page of the calendar to be shown
       *
       * @param {Object} baseDate - baseDate is set to the 1st of the currently selected month
       * @param {Object} startDate - the selected start date
       * @param {Object} endDate - the selected end date
       * @param {Object} selectedDateType - the type of that is being selected: can be start date or end date
       *
       * @returns {Object} weeks the object representing the calendar days to show
       */
      setDateGrid: function (baseDate, startDate, endDate, selectedDateType) {
        var days = this.getDaysToShow(baseDate, startDate, endDate, selectedDateType);
        var weeks = [];
        for (var w = 0; w < 6; w++ ) {
          var week = {days: []};
          for (var d = 0; d < 7; d++ ) {
            var dayElement = days[w * 7 + d];
            if (dayElement.isSelectedDate) {
              dayElement.class = "currentlySelectedDate";
            } else if (dayElement.isInPeriod) {
              dayElement.class = "currentlySelectedPeriod";
            } else if (dayElement.isToday) {
              dayElement.class = "today";
            } else if (dayElement.isNotThisMonth) {
              dayElement.class = "notThisMonth";
            }
            week.days.push(days[w * 7 + d]);
          }
          weeks.push(week);
        }
        return weeks;
      },

      /**
       * @description gets the days in the selected month and year
       *
       * @param {Object} baseDate - baseDate is set to the 1st of the currently selected month
       * @param {Object} startDate - the selected start date
       * @param {Object} endDate - the selected end date
       * @param {Object} selectedDateType - the type of that is being selected: can be start date or end date
       *
       * @returns {Object} days the list of days
       */
      getDaysToShow: function (baseDate, startDate, endDate, selectedDateType) {
        var days = [];
        var firstDay = baseDate.getDay() - 1;
        firstDay = (firstDay < 0) ? 6 : firstDay;
        for (var i = 0; i < firstDay; i ++) {
          var pastDate = new Date(baseDate.getFullYear(), baseDate.getMonth(), baseDate.getDate() - (firstDay - i));
          var inPeriod = false;
          if (this.compareDates(pastDate, startDate) >= 0 && this.compareDates(pastDate, endDate) <= 0) {
            inPeriod = true;
          }
          days.push({date: pastDate, name: pastDate.getDate(), isInPeriod: inPeriod, isNotThisMonth: true});
        }

        for (var j = 0; j < 42 - firstDay; j ++) {
          var date = new Date(baseDate.getFullYear(), baseDate.getMonth(), baseDate.getDate() + j);
          var dateElement = {date: date, name: date.getDate()};
          dateElement.isNotThisMonth = (date.getMonth() !== baseDate.getMonth());
          dateElement.isSelectedDate = false;


          dateElement.isInPeriod = this.compareDates(date, startDate) >= 0 && this.compareDates(date, endDate) <= 0;
          if (selectedDateType === "start" && this.compareDates(date, startDate) === 0 ||
            selectedDateType === "end" && this.compareDates(date, endDate) === 0 ) {
            dateElement.isSelectedDate = true;
          }
          if (this.compareDates(date, new Date()) === 0) {
            dateElement.isToday = true;
          }
          days.push(dateElement);
        }

        return days;
      },

      /**
       * @description transform relative date in absolute date, based on today date
       *
       * @param {Object} relativeDate - the relative start date
       * @param {Object} endRelative - the relative end date
       *
       * @returns {Object} object containing absolute dates
       */
      getDateFromRelative: function (relativeDate) {
        if (!relativeDate){
          return null;
        }
        var type = relativeDate.split('-')[0];
        var value = relativeDate.split('-')[1];
        var date = new Date();
        switch (type) {
          case 'd':
            date.setDate(date.getDate() - value);
            break;
          case 'w':
            var mon = (date.getDay() === 0) ? 6 : date.getDay() % 7 - 1 ;
            date.setDate(date.getDate() - (mon + 7 * value));
            break;
          case 'we':
            var sun = (date.getDay() === 0) ? 0 : 7 - date.getDay();
            date.setDate(date.getDate() - (7 * value) + sun);
            break;
          case 'm':
            date.setDate(1);
            date.setMonth(date.getMonth() - value);
            break;
          case 'me':
            date.setMonth(date.getMonth() - value + 1);
            date.setDate(0);
            break;
          case 'y':
            date.setDate(1);
            date.setMonth(0);
            date.setFullYear(date.getFullYear() - value);
            break;
          case 'ye':
            date.setMonth(0);
            date.setFullYear(date.getFullYear() - value + 1);
            date.setDate(0);
            break;
        }

        date.setHours(0, 0, 0);
        return date;
      },

      /**
       * @description check the two dates (not caring about hours) and returns -1 id the first is smaller, 0 if equals
       * and 1 if bigger
       *
       * @param {Object} date1 - the first date
       * @param {Object} date2 - the second date
       *
       * @returns {Number} integer -1, 0, 1
       */
      compareDates: function (date1, date2) {
        if ($filter('date')(date1, 'yyyy-MM-dd') === $filter('date')(date2, 'yyyy-MM-dd')) {
          return 0;
        } else if (date1 > date2) {
          return 1;
        } else {
          return -1;
        }
      }
    };
  }]);