"use strict";

describe('datePicker Factory', function() {
  beforeEach(module('glDatePickerFactory'));
  var datePicker, $filter, today, start;

  beforeEach(inject(['glDatePickerFactory', '$filter',
    function (GlDatePickerFactory, _$filter) {
      datePicker = GlDatePickerFactory;
      $filter = _$filter
      today = new Date($filter('date')(new Date(), 'yyyy-MM-dd'));
      start = new Date(today);
    }]));

  describe('checkDefaultDates', function () {
    it('should set defaultDates to true when dates match with defaults', function () {
      start.setDate(today.getDate() -7 );
      var end = new Date(today);
      end.setDate(today.getDate() -1 );

      var scope = {startDate: start, endDate: end, settings: {defaults: ["d-7", "d-1" ]}};
      datePicker.checkDefaultDates(scope);
      expect(scope.defaultDates).toEqual(true);
    });

    it("should set defaultDates to false when dates don't match with defaults", function () {
      start.setDate(today.getDate() -9 );
      var end = new Date(today);
      end.setDate(today.getDate() -1 );

      var scope = {startDate: start, endDate: end, settings: {defaults: ["d-7", "d-1" ]}};
      datePicker.checkDefaultDates(scope);
      expect(scope.defaultDates).toEqual(false);
    });

    it("should set defaultDates to true when setting are not defined", function () {
      start.setDate(today.getDate() -9 );
      var end = new Date(today);
      end.setDate(today.getDate() -1 );

      var scope = {startDate: start, endDate: end};
      datePicker.checkDefaultDates(scope);
      expect(scope.defaultDates).toEqual(true);
    });

    it("should set defaultDates to true when default dates are not defined", function () {
      start.setDate(today.getDate() -9 );
      var end = new Date(today);
      end.setDate(today.getDate() -1 );

      var scope = {startDate: start, endDate: end, settings: {defaults: []}};
      datePicker.checkDefaultDates(scope);
      expect(scope.defaultDates).toEqual(true);
    });
  });

  describe('setDateGrid', function () {
    it('should create the day grid for the selected month', function () {
      today.setHours(0);
      var baseDate = new Date(2015, 7, 1);
      var startDate = new Date(2015, 7, 10);
      var endDate = new Date(2015, 7, 15);
      var firstDay = new Date(2015, 6, 27);

      var weeks = datePicker.setDateGrid(baseDate, startDate, endDate, "start");
      expect(weeks[0].days[0]).toEqual({date: firstDay, name: 27, isNotThisMonth: true, isInPeriod: false, class: 'notThisMonth' });
      expect(weeks[2].days[0]).toEqual({date: startDate, name: 10, isNotThisMonth: false, isInPeriod: true, isSelectedDate: true, class: 'currentlySelectedDate'});
      expect(weeks[2].days[5]).toEqual({date: endDate, name: 15, isNotThisMonth: false, isInPeriod: true, isSelectedDate: false, class: 'currentlySelectedPeriod'});
    });
  });

  describe('getDaysToShow', function () {
    it('should return the list of days to to show for the selected month', function () {
      today.setHours(0, 0, 0);
      var baseDate = new Date(2015, 7, 1);
      var startDate = new Date(2015, 7, 10);
      var endDate = new Date(2015, 7, 15);
      var firstDay = new Date(2015, 6, 27);
      var days = datePicker.getDaysToShow(baseDate, startDate, endDate, "start");
      expect(days[0]).toEqual({date: firstDay, name: 27, isNotThisMonth: true, isInPeriod: false});
      expect(days[14]).toEqual({date: startDate, name: 10, isNotThisMonth: false, isInPeriod: true, isSelectedDate: true});
      expect(days[19]).toEqual({date: endDate, name: 15, isNotThisMonth: false, isInPeriod: true, isSelectedDate: false});
    });
  });

  describe('getDateFromRelative', function () {
    it('should get absolute date from day relative date', function () {
      var date = new Date($filter('date')(new Date(), 'yyyy-MM-dd'));
      date.setHours(0);
      date.setDate(date.getDate() - 7);
      var relativeDate = datePicker.getDateFromRelative('d-7');
      expect($filter('date')(relativeDate, 'yyyy-MM-dd')).toEqual($filter('date')(date, 'yyyy-MM-dd'));
    });

    it('should get absolute date from week relative date', function () {
      var date = new Date($filter('date')(new Date(), 'yyyy-MM-dd'));
      date.setHours(0);
      var mon = (date.getDay() === 0) ? 6 : date.getDay() % 7 - 1;
      date.setDate(date.getDate() - (mon + 7));
      var relativeDate = datePicker.getDateFromRelative('w-1');
      expect($filter('date')(relativeDate, 'yyyy-MM-dd')).toEqual($filter('date')(date, 'yyyy-MM-dd'));
    });

    it('should get absolute date from week end relative date', function () {
      var date = new Date($filter('date')(new Date(), 'yyyy-MM-dd'));
      date.setHours(0, 0 ,0);
      var mon = (date.getDay() === 0) ? 6 : date.getDay() % 7 - 1;
      date.setDate(date.getDate() - (mon + 1));
      var relativeDate = datePicker.getDateFromRelative('we-1');
      expect($filter('date')(relativeDate, 'yyyy-MM-dd')).toEqual($filter('date')(date, 'yyyy-MM-dd'));
    });

    it('should get absolute date from month relative date', function () {
      var date = new Date($filter('date')(new Date(), 'yyyy-MM-dd'));
      date.setHours(0, 0 ,0);
      date.setDate(1);
      date.setMonth(date.getMonth() - 1);
      var relativeDate = datePicker.getDateFromRelative('m-1');
      expect($filter('date')(relativeDate, 'yyyy-MM-dd')).toEqual($filter('date')(date, 'yyyy-MM-dd'));
    });

    it('should get absolute date from month end relative date', function () {
      var date = new Date($filter('date')(new Date(), 'yyyy-MM-dd'));
      date.setHours(0, 0 ,0);
      date.setDate(0);
      date.setMonth(date.getMonth());
      var relativeDate = datePicker.getDateFromRelative('me-1');
      expect($filter('date')(relativeDate, 'yyyy-MM-dd')).toEqual($filter('date')(date, 'yyyy-MM-dd'));
    });

    it('should get absolute date from year relative date', function () {
      var date = new Date($filter('date')(new Date(), 'yyyy-MM-dd'));
      date.setHours(0, 0 ,0);
      date.setDate(1);
      date.setFullYear(date.getFullYear() - 1);
      date.setMonth(0);
      var relativeDate = datePicker.getDateFromRelative('y-1');
      expect($filter('date')(relativeDate, 'yyyy-MM-dd')).toEqual($filter('date')(date, 'yyyy-MM-dd'));
    });

    it('should get absolute date from year end relative date', function () {
      var date = new Date($filter('date')(new Date(), 'yyyy-MM-dd'));
      date.setHours(0, 0 ,0);
      date.setMonth(0);
      date.setDate(0);
      var relativeDate = datePicker.getDateFromRelative('ye-1');
      expect($filter('date')(relativeDate, 'yyyy-MM-dd')).toEqual($filter('date')(date, 'yyyy-MM-dd'));
    });

    it('when relative date passed is null should return null', function () {
      var relativeDate = datePicker.getDateFromRelative(null);
      expect(relativeDate).toEqual(null);
    });
  });

  describe('compareDates', function () {
    it('when dates describe the same day should return 0', function () {
      var date1 = new Date();
      var date2 = new Date();
      date2.setHours(12, 12, 12);
      var comparison = datePicker.compareDates(date1, date2);
      expect(comparison).toEqual(0);
    });
    it('when first date is one or more days earlier than second date should return -1', function () {
      var date1 = new Date();
      var date2 = new Date();
      date1.setDate(date1.getDate() - 1);
      var comparison = datePicker.compareDates(date1, date2);
      expect(comparison).toEqual(-1);
    });
    it('when first date is one or more days later than second date should return 1', function () {
      var date1 = new Date();
      var date2 = new Date();
      date1.setDate(date1.getDate() + 1);
      var comparison = datePicker.compareDates(date1, date2);
      expect(comparison).toEqual(1);
    });
  });
});